

/*
 * TalkClient.java
 * 
 * Client portion of the chat room program.
 * Accepts input from the user and sends it to the server and
 * recieve the output of the server.
 *
 * Created on September 16, 2003, 2:45 AM
 */

/**
 *
 * @author   Nathan Balon
 * SN 3210 1717
 * CIS 527
 * Program Assignment #1
 * 
 *
 */

import java.net.*;
import java.io.*;
import java.util.*;
//import java.util.regex.*;

public class TalkClient extends Thread{
        
    private final static short SERVER_PORT = 1717; // port # of chat server
    private boolean loggedIn = false;              // if the user is logged in   
    private BufferedReader serverInputStream;      // input stream from server    
    private Socket clientSocket;                   // client socket 
    private BufferedReader is;                     // input stream from terminal         
    private PrintWriter os;                        // output stream    
    private String userCommand;                    // commands entered from user
    private boolean continueProgram = true;        // flag to keep program running
    private User currentUser = null;               // User of that is logged in


    /** TalkClient constructor */    
    public TalkClient(Socket s, BufferedReader serverIStream, 
                       BufferedReader iStream, PrintWriter oStream){
        
        clientSocket = s;
        serverInputStream = serverIStream;
        is = iStream;
        os = oStream;

    }
    
    /** Displays to the user a list of availiable commands.  */
    public static void help(){
        System.out.println("The availiable commands are:  login, send, " +
                           "who, new, logout, quit");
    }
    

    /* Check if user is logged in */
    public boolean isLoggedIn(){
        if (loggedIn == true)
            return true;
        else
            return false;      
    }

    
    /** log the user on or off the chat program*/
    public void setLogin(boolean b){
        loggedIn = b;
    }

    
    /** Prompts user to enter their name and password and makes sure
     *  that they are in the correct format                        */
    private String enterNamePass(BufferedReader is) throws IOException{
        String nameAndPass = "";               // users name and pass
        String password = "";                  // user password
        
        System.out.print("UserID: ");
        String userName = is.readLine();
                        
        // Check if userID is the correct length if not
        // prompt them to enter it until it is correct.
        while(! TalkClient.checkStringLength(userName)){
            System.out.print("UserID: ");
            userName = is.readLine();
        }
        nameAndPass += " " + userName;
            
        // enter password
        System.out.print("Password: ");
        password = is.readLine();
                        
        // Check if the password is the correct length if not
        // if not prompt the user to enter it until it is correct.
        while(! TalkClient.checkStringLength(password)){
            System.out.print("Password: ");
            password = is.readLine();
        }
        // add the password string to the name 
        nameAndPass += " " + password;
        // convert the string to lower case
        nameAndPass = nameAndPass.toLowerCase(); 
        return nameAndPass;
    }

    /**  log the user into the chat program        */
    public void login(String userCommand, BufferedReader is, PrintWriter os)
                                                           throws IOException{
        String userName = "";                  // user name
        String password = "";                  //password
 
        // Check if user is already logged in
        if(this.isLoggedIn() == true){
            System.out.println("User is already logged in");
        }
                     
        // user did not enter useranme and password prompt them
        else{
            // enter userID
            userCommand += this.enterNamePass(is);

            // send loggin information to the server
            os.println(userCommand);
        }
    } 
    

    
    /** Check the string length so it is between 3 and 8 characters 
        the function returns true if it meets the requirements   */
    private static boolean checkStringLength(String s){
        if((s.length() > 8) || (s.length() < 3)){
            System.out.println("The length of the string needs to " +
                               "be between 3 and 8 characters.");
            // return false if less the 3 or more then 8 characters
            return false;      
                              
        }
        else {
            return true;    // return true if between 3 and 8 characters
        }
    }    
    
    public void run(){
        while(continueProgram == true){
           try{
            String command = " ";
            System.out.print("Chat> "); 
            // read the command from the client
            userCommand = is.readLine();
            // convert the command to lower case
            userCommand = userCommand.toLowerCase();
            // create a StringTokenizer to get client commands
            StringTokenizer commandTok = new StringTokenizer(userCommand);
            if(commandTok.hasMoreTokens()){
                command = commandTok.nextToken();
            }
                 
            // Check if user entered send command                
            if(command.equals("send")){
                if(this.isLoggedIn() == false){     
                    System.out.println("Permission denied, you " +
                                       "must login first.");
                }
                else{
                    // accept message from user to be sent
                    System.out.print("To User: ");
                    String user = is.readLine();
                    System.out.print("Enter Message: ");
                    userCommand = is.readLine();
                    // send the message to the server
                    os.println("send " +  user + " " + currentUser.getName() +
                               " " + userCommand); 
                    // read message from the server
                    //System.out.println(serverInputStream.readLine());
                }
            }
                 
            // check if user entered who command
            else if(command.equals("who")){
                if(this.isLoggedIn() == true){
                    os.println("who");
                    System.out.println(serverInputStream.readLine());
                }
                else{
                    System.out.println("!! Please login first !!");
                }
            }
                
            // Check if user enter login as the command
            else if(command.equals("login")){
            // check if a user is already logged in
                if(this.isLoggedIn()){
                    System.out.println("User is already logged in");
                }
                // login the user if they supplied arguement to 
                // the send commad
                else if(commandTok.hasMoreTokens()){
                    // get the users name
                    String name = commandTok.nextToken();
                    // check that the name is the correct length
                    if(this.checkStringLength(name)){
                        if(commandTok.hasMoreTokens()){
                            // get the users password
                            String pass = commandTok.nextToken();
                            // check that the password is the correct length
                            if(this.checkStringLength(pass)){
                                // create a new user
                                currentUser = new User(name,pass);
                                // send user info to server to be logged in
                                os.println("login " + name + " " + pass);
                                String inputStream = serverInputStream.readLine();
                                if(inputStream.startsWith("true")){
                                    this.setLogin(true);
                                    System.out.println("SERVER# " + name +  " joined the chat room");
                                }
                                else{
                                    System.out.println(name + 
                                                " could not be logged in");
                                }                      
                            }
                            else{
                                System.out.println("Password incorrect format");
                            }
                        }
                        else{
                            System.out.println("You must supply the password");
                        }
                   }
                    else{
                        System.out.println("!! Name incorrect format !!");
                    }
		}
                else{     
                    // log the user in to the chat room
                    this.login(userCommand, is, os);
                    String inputStream = serverInputStream.readLine();
                    StringTokenizer tok =
                            new StringTokenizer(inputStream);
                        
                    // get true or false is user is logged in
                    String clientInput = tok.nextToken(); 
                        
                    // if client is logged in the server
                    if(clientInput.equals("true")){
                        // get the name and pass of the
                        // logged in user from the server
                        String name = tok.nextToken();
                        String pass = tok.nextToken();
                        // create the logged in user
                        currentUser = new User(name, pass);
                        // set the user to logged in
                        this.setLogin(true);
                        while(tok.hasMoreTokens()){
                            System.out.print(tok.nextToken() + " ");
                        }

                    }
                    else{
                        // diplay message why user can't be logged in
                        while(tok.hasMoreTokens()){
                            System.out.print(tok.nextToken() + " ");
                        }
                    }
                            // end the line
                    System.out.println("");
                }
            }
            
            // Check if user entered logout
            else if(command.equals("logout")){
                // if no users are logged in 
                if(this.isLoggedIn() == false){
                    System.out.println("You are not logged in");
                }
                // log the user out so they can't send any messages
                else{
                    // send message to server to log them out
                    os.println("logout " + currentUser.getName()
                                   + " " + currentUser.getPassword());
                    System.out.println("SERVER# " + 
                                            currentUser.getName() +
                                            " logged out of chat");
                    currentUser.setName("");
                    // log the user out of chat on the client
                    this.setLogin(false);
                }
            }
                 
                 // Check if user entered
            else if(command.equals("quit")){
                // if user is logged in send a message to server to log them out
                if(this.isLoggedIn()){
                    os.println("logout " + currentUser.getName()
                               + " " + currentUser.getPassword());
                }
                
                continueProgram = false;
                os.println("quit");
                System.out.println("!! Program Terminated !!");
                System.exit(0);
                
            }
            
            else if(command.equals("new")){
                System.out.println("Enter new user information");
                // function to prompt user for info
                userCommand += this.enterNamePass(is);
                // send new user info to the server
                os.println(userCommand);         
                // display message from the server
                System.out.println(serverInputStream.readLine());    
            }
                
            // Check if user entered help and display commands
            else if(command.equals("help")){
                TalkClient.help();
            }
                 
            // if none of the commands display an error command 
            else {
                System.out.println("!! Error Unknown Command !!");
            }        
        //  }
           }  
          catch(IOException e){
              e.printStackTrace();
          }
        }
    }   
    
    /** Main entry point to run the TalkClient application */
    public static void main(String args[]){       
        // check to make sure that the user entered the IP address 
        if (args.length != 1){
            System.err.println("\nThe IP address must be " + 
                               "specified as an arguement\n");
        }        
        try{
            // create a new socket
            Socket s = new Socket(args[0], SERVER_PORT);

            // Create a BufferedReader to read input commands from the user.
            BufferedReader iStream = new BufferedReader(new InputStreamReader(System.in)); 
            BufferedReader serverIStream = new BufferedReader(new InputStreamReader(
                                                    s.getInputStream()));

            // Create a PrintWriter to recieve data from the server.
            PrintWriter oStream = new PrintWriter(s.getOutputStream(), true);     
            
            TalkClient talkClient = new TalkClient(s, serverIStream, iStream, oStream);
            talkClient.setPriority(5);
            // create a new thread to read messages to all users
            TalkClientReadSocket readClient = new TalkClientReadSocket(s);
            readClient.setPriority(3);
            
            // Display the program start up output
            System.out.println("Connection Created to: " +
                          InetAddress.getByName(args[0]).getHostName());
            System.out.println("Internet Chat Room");
            System.out.println("Type \"help\" for a list of commands.");
            
            // start the threads 
            talkClient.start();
            readClient.start();
            
            // if talkClient thread is not alive destroy the read thread
            if(!talkClient.isAlive()){
                readClient.destroy();
            }
        }       
        catch (IOException e){
            System.err.println("IO ERROR occured: " + e);
       }
       catch(Exception e){
            System.err.println("Error: " + e);
       }
    }
}
